"""
Use 303 and 307 instead of 302, to deal with faulty browser implementations. See
http://stackoverflow.com/questions/4764297/difference-between-http-redirect-codes
for more on the issue.

"""

from django.http import HttpResponseRedirect


class Http303(HttpResponseRedirect):
    status_code = 303


class Http307(HttpResponseRedirect):
    status_code = 307



from django.core.exceptions import ImproperlyConfigured

from .requirements import BaseRequirement


class ManagedAccessViewMixin(object):
    """
    Runs checks for requirements before running `dispatch` of the subclass.
    The subclass needs to specify `access_requirements` as an iterable of
    requirements.

    """

    access_requirements = None

    def get_access_requirements(self):
        if self.access_requirements is None:
            raise ImproperlyConfigured(
                "Views that extends ManagedAccessViewMixin need to specify "
                "`access_requirements` or implement a "
                "`get_access_requirements` method.")
        return self.access_requirements

    def dispatch(self, *args, **kwargs):
        klasses = self.get_access_requirements()

        for requirement in klasses:
            if not isinstance(requirement, BaseRequirement):
                requirement = requirement()
            requirement.setup(*args, **kwargs)
            if not requirement.is_fulfilled():
                return requirement.not_fulfilled()

        return super(ManagedAccessViewMixin, self).dispatch(*args, **kwargs)



from django.core.exceptions import ImproperlyConfigured
from django.core.urlresolvers import reverse
from django.http import Http404

from .http import Http307


class BaseRequirement(object):
    def setup(self, request, *args, **kwargs):
        self.request = request
        self.args = args
        self.kwargs = kwargs

    def is_fulfilled(self):
        raise ImproperlyConfigured(
            "Requirements need to implement a `is_fulfilled` method.")

    def not_fulfilled(self):
        raise ImproperlyConfigured(
            "Requirements need to implement a `not_fulfilled` method.")


class BasePageNotFoundRequirement(BaseRequirement):
    def not_fulfilled(self):
        raise Http404()


class Staff(BasePageNotFoundRequirement):
    def is_fulfilled(self):
        return self.request.user.is_staff


class SuperUser(BasePageNotFoundRequirement):
    def is_fulfilled(self):
        return self.request.user.is_superuser


class Active(BasePageNotFoundRequirement):
    def is_fulfilled(self):
        return self.request.user.is_active


class BaseRedirectRequirement(BaseRequirement):
    url_name = None
    append_next = True

    def get_url_args(self):
        return []

    def get_url_name(self):
        if self.url_name is None:
            raise ImproperlyConfigured(
                "You need to specify `url_name` or override the `get_url_name` "
                "method.")
        return self.url_name

    def get_url(self):
        url = reverse(self.get_url_name(), args=self.get_url_args())
        if self.append_next:
            url += "?next=%s" % self.request.get_full_path()
        return url

    def not_fulfilled(self):
        return Http307(self.get_url())


class LoggedIn(BaseRedirectRequirement):
    url_name = 'login'

    def is_fulfilled(self):
        return self.request.user.is_authenticated()



from django.test import TestCase

from access_manager.views import ManagedAccessViewMixin
from access_manager.requirements import BaseRequirement


class SuccessfulRequirement(BaseRequirement):
    not_fulfilled_called = False
    is_fulfilled_called = False
    return_value = True

    def not_fulfilled(self):
        self.not_fulfilled_called = True

    def is_fulfilled(self):
        self.is_fulfilled_called = True
        return self.return_value


class UnSuccessfulRequirement(SuccessfulRequirement):
    return_value = False


class View(object):
    dispatch_called = False

    def dispatch(self, *args, **kwargs):
        self.dispatch_called = True


class FakeView(ManagedAccessViewMixin, View):
    pass


class TestManagedAccessViewMixin(TestCase):
    def setUp(self):
        self.view = FakeView()
        self.request = {}

    def test_successful(self):
        first = SuccessfulRequirement()
        second = SuccessfulRequirement()
        self.view.access_requirements = [
            first, second]
        self.view.dispatch(self.request)
        self.assertTrue(self.view.dispatch_called)
        self.assertTrue(first.is_fulfilled_called)
        self.assertTrue(second.is_fulfilled_called)

    def test_first_unfulfilled(self):
        first = UnSuccessfulRequirement()
        second = SuccessfulRequirement()
        self.view.access_requirements = [first, second]
        self.view.dispatch(self.request)
        self.assertFalse(self.view.dispatch_called)
        self.assertTrue(first.is_fulfilled_called)
        self.assertTrue(first.not_fulfilled_called)
        self.assertFalse(second.is_fulfilled_called)
        self.assertFalse(second.not_fulfilled_called)

    def test_second_unfulfilled(self):
        first = SuccessfulRequirement()
        second = UnSuccessfulRequirement()
        self.view.access_requirements = [first, second]
        self.view.dispatch(self.request)
        self.assertFalse(self.view.dispatch_called)
        self.assertTrue(first.is_fulfilled_called)
        self.assertFalse(first.not_fulfilled_called)
        self.assertTrue(second.is_fulfilled_called)
        self.assertTrue(second.not_fulfilled_called)



from django.test import TestCase
from django.http import HttpRequest
from django.contrib.auth.models import AnonymousUser

from access_manager.requirements import (
    Staff, SuperUser, LoggedIn, Active)
from .factories import UserFactory, InActiveUserFactory


class TestRequirements(TestCase):
    request = None

    def setUp(self):
        self.request = None

    def set_request(self, **kwargs):
        self.request = HttpRequest()
        for k, v in kwargs.items():
            setattr(self.request, k, v)

    def assert_requirement_fulfilled(self, requirement):
        requirement.setup(self.request)
        self.assertTrue(
            requirement.is_fulfilled(), msg="The requirement is not fulfilled")

    def assert_requirement_unfulfilled(self, requirement):
        requirement.setup(self.request)
        self.assertFalse(
            requirement.is_fulfilled(), msg="The requirement is fulfilled")

    def test_staff(self):
        user = UserFactory.build(is_staff=False)
        self.set_request(user=user)
        self.assert_requirement_unfulfilled(Staff())

        user.is_staff = True
        self.assert_requirement_fulfilled(Staff())

    def test_superuser(self):
        user = UserFactory.build(is_superuser=False)
        self.set_request(user=user)
        self.assert_requirement_unfulfilled(SuperUser())

        user.is_superuser = True
        self.assert_requirement_fulfilled(SuperUser())

    def test_active(self):
        user = InActiveUserFactory.build()
        self.set_request(user=user)
        self.assert_requirement_unfulfilled(Active())

        user.is_active = True
        self.assert_requirement_fulfilled(Active())

    def test_logged_in(self):
        self.set_request(user=AnonymousUser())
        self.assert_requirement_unfulfilled(LoggedIn())

        self.set_request(user=UserFactory.build())
        self.assert_requirement_fulfilled(LoggedIn())



from django.contrib.auth.models import User
from django.contrib.auth.hashers import make_password

import factory


class InActiveUserFactory(factory.DjangoModelFactory):
    FACTORY_FOR = User

    username = factory.Sequence(lambda n: 'user{0}'.format(n))
    first_name = "Bill"
    last_name = "Murray"
    is_active = False
    is_superuser = False
    is_staff = False
    email = "anemail@example.com"
    password = make_password("password")


class UserFactory(InActiveUserFactory):
    is_active = True



#!/usr/bin/env python

from setuptools import setup, find_packages
import access_manager

setup(
    name='django-access-manager',
    version=".".join(map(str, access_manager.__version__)),
    author='Anton Agestam',
    author_email="msn@antonagestam.se",
    url='http://github.com/FundedByMe/django-access-manager',
    install_requires=[
        'Django>=1.4.8',
    ],
    description='An abstract access manager for Django.',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        "Framework :: Django",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Operating System :: OS Independent",
        "Topic :: Software Development"
    ],
)
